/* $Id: euro2.c,v 1.1 1998/03/12 20:43:15 svmcguir Exp $ */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "path.h"
#include "option.h"
#include "mesh.h"
#include "util.h"

/* Calculates the values of the european option corresponding to the
 * american option we are evaluating. Values returned in  results[time]
 * are the european values at each timeslice.
 */
void euro2(meshData *md, double tolerance, double *result){
  int time, underlier, count;
  double **path, **antiPath, payoff, antiPayoff, *sum, *sumSq, *relError, variance, value, maxError;
  
  allocate2dDouble(&path,  md->option.pData.times,  md->option.pData.underliers);
  allocate2dDouble(&antiPath,  md->option.pData.times,  md->option.pData.underliers);
  allocate1dDouble(&sum, md->option.pData.times);
  allocate1dDouble(&sumSq, md->option.pData.times);
  allocate1dDouble(&relError, md->option.pData.times);

  for(time=0; time < md->option.pData.times; time++){
    sum[time] = 0;
    sumSq[time] = 0;
  }

  for(count=0; count < 1000; count+=2){
    pathPair(&(md->option.pData),path,antiPath);

    for(time=0; time < md->option.pData.times; time++){
      payoff = dMax(0,exp(-time*md->option.riskFreeRate)*optionPayoff(&(md->option), path[time]));
      antiPayoff = dMax(0,exp(-time*md->option.riskFreeRate)*optionPayoff(&(md->option), antiPath[time]));
      sum[time] += payoff + antiPayoff;
      sumSq[time] += payoff*payoff + antiPayoff*antiPayoff;
    }
  }

  do{
    pathPair(&(md->option.pData),path,antiPath);

    for(time=0; time < md->option.pData.times; time++){
      payoff = dMax(0,exp(-time*md->option.riskFreeRate)*optionPayoff(&(md->option), path[time]));
      antiPayoff = dMax(0,exp(-time*md->option.riskFreeRate)*optionPayoff(&(md->option), antiPath[time]));
      sum[time] += payoff + antiPayoff;
      sumSq[time] += payoff*payoff + antiPayoff*antiPayoff;
    } 
    
    count += 2;

    maxError = 0;
    for(time=0; time < md->option.pData.times; time++){
      value = sum[time]/count;
      variance  = sumSq[time]/count - sum[time]/count*sum[time]/count;
      relError[time] = sqrt(variance/count)/fabs(value);
      if(relError[time] > maxError) maxError = relError[time];
    }
  }while(maxError > tolerance);


  printf("count = %i\n",count);
  for(time=0; time < md->option.pData.times; time++){
    result[time] = sum[time]/count;
  }
}

